/*
 * Component:  utils
 *
 * Purpose:
 *    Header file for the double chained list implementation
 *
 * Last Updated     : $Author$
 * Update Date Time : $Date$
 * RCS File Name    : $RCSfile$
 * RCS Revision     : $Revision$
 * Locked by        : $Locker$
 * Symbolic name    : $Name$ 
 * State            : $State$
 *
 */

#ifndef LIST_H
#define LIST_H

#include <stdlib.h>

class List;

/**
* Base class for elements of a double chained list
*/
class ListElem
{
friend class List;

public:

   /**
   * Default constructor
   */
	ListElem();

	/**
	* Destructor
	*/
	virtual ~ListElem();

private:

	/**
	* Gets previous element from the list
	*
	* @return the previous element
	*/
	inline ListElem * getPrev() { return m_prev;	}

	/**
	* Gets next element in the list
	*
	* @return the next element
	*/
	inline ListElem * getNext() {	return m_next;	}

	/**
	* Sets previous element
	*
	* @param prev   Set the previous pointer to this ListElem
	*/
	inline void setPrev(ListElem * prev) {	m_prev = prev;	}

	/**
	* Sets next element
	*
	* @param prev   Set the next pointer to this ListElem
	*/
	inline void setNext(ListElem * next) {	m_next = next;	}

	/** Pointer to the next element in the list */
	ListElem * m_next;

	/** Pointer to the previous element in the list */
	ListElem * m_prev;

protected:

	/**
	* Checks if a ListElem is the same as this ListElem:
	* This base implementation will only check pointers;
	*
	* return true if and only if elem is equals to this.
	*/
	virtual bool equals(ListElem & elem);
};



/**
* A double chained list implementation
*/
class List
{
private:
	/**
	* Head of the list. Head element will contain no data but has a list
	* management function.
	*/
	ListElem * m_head;

public:

	/**
	* Default constructor for creating new List object
	*/
	List();

	/**
	* Constructor for creating new list with a given head element.
	* Recommend to use this constructor for creating lists when find()
	* functionality will be used in the list.
	*
	* @param head   The head element for the list.
	*/
	List(ListElem * head);

	/**
	* Destructor
	*/
	~List();

	
	/**
	* Retrieves the next element of a specified element in the list
	*
	* @param elem  get the next element of this element
	*
	* @return the next element in the list
	*/
	ListElem * getNext(ListElem * elem);

	/**
	* Retrieves the previous element of a specified element in the list
	*
	* @param elem  get the previous element of this element
	*
	* @return the previous element in the list
	*/
	ListElem * getPrev(ListElem * elem);

	/**
	* Removes an element from the list. Returns the next/previous element in the
	* list regarding to the returnPrev parameter.
	*
	* @param elem         The element to be removed from the list
	* @param returnPrev   true indicates that return with the previous element (default is next)
	*
	* @return the next/previous element in the list regarding to the value of the returnPrev parameter
	*/
	ListElem * remove(ListElem * elem, bool returnPrev = false);

	/**
	* Puts an element to the list. The element will be inserted as the last element in
	* the list unless the value of the addFirst parameter is true, in wich case the
	* element will be inserted as the first element in the list.
	*
	* @param elem      the element to be inserted
	* @param addFirst  true indicates that the elem will be the first element in the list
	*/
	void put(ListElem * elem, bool addFirst = false);

	/**
	* Puts an element to the list. The element will be inserted after a given element (pos) in the list.
	*
	* @param elem      the element to be inserted
	* @param pos       place the element after this element
	*/
	void putAfter(ListElem * elem, ListElem * pos);

	/**
	* Puts an element to the list. The element will be inserted before a given element (pos) in the list.
	*
	* @param elem      the element to be inserted
	* @param pos       place the element before this element
	*/
	void putBefore(ListElem * elem, ListElem * pos);

	/**
	* Finds the first list element matching to the find criterions
	* set in the head element (usually ID fields).
	*
	* @return the matching list element or NULL if not found.
	*/
   ListElem * find();

	/**
	* Checks if the list is empty or not
	*
	* @return true if and only if the list is empty.
	*/
	inline bool isEmpty() {
		return (m_head == NULL) || (m_head->getNext() == m_head);
	}

   /**
   * Retrieves the head element of the list
   *
   * @return the head element of the list
   */
   inline ListElem * getHead() {
		return m_head;
   }

};
#endif
